@@ -517,16 +517,22 @@ class Dated(PathData):
 
     def __lt__(self, other):
         if self.date == other.date:
+            if self.time == other.time:
+                return self.path < other.path
             return self.time < other.time
         return self.date < other.date
 
     def __gt__(self, other):
         if self.date == other.date:
+            if self.time == other.time:
+                return self.path > other.path
             return self.time > other.time
         return self.date > other.date
 
     def __le__(self, other):
         if self.date == other.date:
+            if self.time == other.time:
+                return self.path < other.path
             return self.time < other.time
         return self.date < other.date
 
@@ -542,9 +548,10 @@ class MachineData(Dated):
 class Archive(Dated):
     """Dated object associated with multiple files."""
 
-    def __init__(self, path: str = "", allow_dirs: bool = False):
+    def __init__(self, path: str = "", auto_timestamp=False,
+            allow_dirs: bool = False):
 
-        Dated.__init__(self, path)
+        Dated.__init__(self, path, auto_timestamp)
 
         # Find names of files within the directory
         self.files = []
@@ -570,8 +577,8 @@ class File(Dated):
     """File with an associated date. Files can be sorted based on their
     filenames."""
 
-    def __init__(self, path: str = ""):
-        Dated.__init__(self, path)
+    def __init__(self, path: str = "", auto_timestamp=False):
+        Dated.__init__(self, path, auto_timestamp)
 
     def __eq__(self, other):
         return self.path == other.path
